<?php
/*
Plugin Name: Recent Commentary Widget
Plugin URI: http://tadhg.com/wp/wordpress-plugins/recent-commentary-widget/
Description: Shows list of posts with recent comments.
Version: 1.0
Author: Tadhg O'Higgins
Author URI: http://tadhg.com/
*/
/*  Copyright 2009 Tadhg O'Higgins  (email : wp-plugins@tadhg.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

Class TadhgRecentCommentary {
    /*
    This class controls the Recent Commentary widget.
    Class variables:
    * $posts: an array of the posts, each post containing an array of the
        comments on that post.
    * $output: the output, normally HTML for the sidebar widget, Other output
        is untested.
    * $options: parameters for controlling widget behavior.
    
    Class functions:
    * __construct: sets up the object, calls the function that handles the
        database queries or if cached gets them from cache, and then calls the
        function that constructs the output.
        Places the database results and the outpout in the $posts and $output
        class variables respectively.
    * recent_commentary: does the database queries and returns an array of
        posts, each post containing an array of the comments on that post.
    * build_*: functions for building the various output formats. Can
        theoretically be called as non-class functions without an instance of
        the class.
    * get_*: functions calling the various build functions; can only be called
        from an instance of the class.
    * widget_control: function controlling the widget admin control panel.
    
    Non-Class convenience functions:
    These are necessary for passing to add_action,
    wp_register_widget_control, and wp_register_sidebar_widget.
    * wp_tadhg_recent_commentary_register: registers the widget for the sidebar
        and the admin panel.
    * wp_tadhg_recent_commentary_control: simply calls widget_control().
    * wp_tadhg_recent_commentary: creates a class instance, calling the
        constructor, and then prints the contents of the instance's output
        variable.
    * wp_tadhg_delete_recent_commentary_cache: clears the cache of the comments.
        
    Typical usage/execution cycle:
    __construct() either gets the data from the cache or calls
    recent_commentary() to handle the database query, then stores the results
    in $posts, then passes those results to build_widget(), which calls
    build_html(), returns the HTML for the list, the HTML is stored in $output,
    and $output is printed.
    */
    public $posts;
    public $output;
    public $options = array(
        "mode"=>"HTML",
        "widget_args"=>array(
            "name" => "Sidebar 1",
            "id" => "sidebar-1",
            "before_widget" =>"",
            "after_widget" =>"",
            "before_title" =>"",
            "after_title" =>"",
            "widget_id" => "related_posts_by_tags",
            "widget_name" => "Related Posts"
        )
    );
    
    function __construct($options=FALSE) {
        //Handle options:
        if ($options) {
            $this->options = array_merge($this->options, $options);
        }
        //Check for cached data:
        if ( !$this->posts = wp_cache_get( 'wp_tadhg_recent_commentary', 'widget' ) ) {
            //If we don't have the cache, get the posts with recent comments and
            //add them to the cache:
            $this->posts = $this->recent_commentary($this->options);
            wp_cache_add( 'wp_tadhg_recent_commentary', $this->posts, 'widget' );
        }
        
        $modefuncs = array(
            //The dict for what function is called by what mode. The awkward
            //structure is required because we're going to end up using
            //call_user_func.
            "HTML"=>array($this, 'get_html'),
            "raw"=>array($this, 'get_raw'),
            "JSON"=>array($this, 'get_json'),
            "src"=>array($this, 'get_src'),
            "widget"=>array($this, 'get_widget')
        );
        $this->output = call_user_func($modefuncs[$this->options["mode"]], $this->posts);
    }
    
    public function recent_commentary($options=array()) {
        /*
        Returns an array of post objects, each containing an array of recent comments.
        */
        //Get the options set by the admin screen and the database access class:
        $admin_options = get_option('widget_tadhg_recent_commentary');
        global $wpdb;
        //Write the passed-in options over the admin ones:
        $options = array_merge($admin_options, $options);
        //Make sure the relevant options make sense:
        if (!$options['postlimit'] || $options['postlimit'] < 1) {
            $options['postlimit'] = 10;
        }
        if ($options['dateorder'] != "ASC" && $options['dateorder'] != "DESC") {
            $options['dateorder'] = "DESC";
        }
        if (isset($this)) {
            $this->options = $options;
        }
        //Get the ids we need for the main query, that is, the posts with the
        //most recent comments:
        $postid_query = "
            SELECT $wpdb->posts.ID, MAX($wpdb->comments.comment_date) as datesort
            FROM $wpdb->comments
            LEFT JOIN $wpdb->posts ON ($wpdb->comments.comment_post_ID = $wpdb->posts.ID)
            WHERE $wpdb->posts.post_status = 'publish'
                AND $wpdb->posts.post_type IN ('post', 'page')
                AND $wpdb->comments.comment_approved = 1
            GROUP BY $wpdb->posts.ID
            ORDER BY datesort DESC
            LIMIT " . $options['postlimit'] .";
        ";
        $post_objects = $wpdb->get_results($postid_query);
        $post_ids = array();
        foreach ($post_objects as $post) {
            array_push($post_ids, $post->ID);
        }
        $comma_separated = implode(",", $post_ids);
        
        //Get the post and comment data for all of the comments on the posts
        //returned by the previous query:
        $recent_commentary_query = "
            SELECT *
            FROM $wpdb->posts
            LEFT JOIN $wpdb->comments ON ($wpdb->posts.ID = $wpdb->comments.comment_post_ID)
            WHERE $wpdb->posts.ID IN ($comma_separated)
                AND $wpdb->comments.comment_approved = 1
            ORDER BY field($wpdb->posts.ID, $comma_separated), $wpdb->comments.comment_date;
        ";
        $recent_commentary_results = $wpdb->get_results($recent_commentary_query);
        
        //Create an array that will store basic post data in each entry, with
        //each entry also containing an array with all of the data about each
        //comment. We need this new structure to deal with per-post comments
        //later.
        $recent_commentary = array();
        
        foreach ($post_ids as $key=>$value) {
            //Go through each post, extract the data relevant to it from the
            //query results, and add it to the array we're going to return.
            $postdata = array(
                "id"=>$value,
                "comments"=>array()
            );
            foreach ($recent_commentary_results as &$row) {
                //If the row matches the current post, insert the title, the
                //comment count, and add the row itself to the list of comments
                //on that post.
                if ($value == $row->ID) {
                    $postdata["title"] = $row->post_title;
                    $postdata["comment_count"] = $row->comment_count;
                    array_push($postdata["comments"], $row);
                }
            }
            array_push($recent_commentary, $postdata);
        }
        
        return $recent_commentary;
    }
    
    //The functions that build the various output:
    public function build_html($posts) {
        /*
        Returns a list in HTML with this structure:
        <ul class="recent-commentary-list">
            <li class="recent-commentary-post">
                <a title="n on post title" href="comments_link"><span class="comment-number">n</span> on post title</a>
                <ul class="recent-commentary-commenters-list">
                    <li><a title="commenter on post title at timestamp" href="comment_link">commenter</a></li>,
                    [repeat for number of commenters]
                </ul>
            </li>
            [repeat for number of posts]
        </ul>
        */
        //Make sure we have some reasonable value for $commentlimit:
        $commentlimit = 10;
        $defaultcss = "";
        if (isset($this)) {
            if (self::pydictget($this->options, 'commentlimit', FALSE) === FALSE) {
                $this->options['commentlimit'] = $commentlimit;
            }
            $commentlimit = (int)($this->options['commentlimit']);
            if (self::pydictget($this->options, 'defaultcss', FALSE)) {
                $defaultcss = "<style type='text/css'>
                    .recent-commentary-commenters-list {
                        display: block;
                    }
                    .recent-commentary-commenters-list li {
                        display: inline;
                        border: 0 !important;
                    }
                </style>";
            }
        }
        $tags_html = $defaultcss . '<ul class="recent-commentary-list">';
        foreach ($posts as &$post) {
            $tags_html .= '<li class="recent-commentary-post">';
            if ($post["comment_count"] == 1) {
                //Posts with just one comment have a different format:
                $text = sprintf('<span class="single-commenter">%s</span> on %s', $post["comments"][0]->comment_author, get_the_title($post["id"]));
                $timestamp = date("H:i d M Y", strtotime($post["comments"][0]->comment_date));
                $title_text = attribute_escape(sprintf("%s at %s", $text, $timestamp));
                $tags_html .= sprintf('<a href="%s#comments" title="%s">%s</a>', get_permalink($post["id"]), $title_text, $text);
            } else {
                $text = sprintf('<span class="comment-number">%s</span> on %s', $post["comment_count"], get_the_title($post["id"]));
                $tags_html .= sprintf('<a href="%s#comments" title="%s">%s</a>', get_permalink($post["id"]), attribute_escape(strip_tags($text)), $text);
                //If there are more commenters than the limit, reflect this with an ellipsis:
                if ($commentlimit > 0 ) {
                    $commenters = ((int)($post["comment_count"]) > $commentlimit) ? "... " : "";
                    foreach ($post["comments"] as $count=>$comment) {
                        //Go through the list of commenters and add them to the
                        //list, but only if they're in the last n comments, where
                        //n is $commentlimit:
                        if ($count +1 > (int)($post["comment_count"]) -$commentlimit ) {
                            $timestamp = date("H:i d M Y", strtotime($comment->comment_date));
                            $title_text = attribute_escape(sprintf("%s on %s at %s", $comment->comment_author, get_the_title($post["id"]), $timestamp));
                            $commenters .= sprintf('<li><a href="%s" title="%s">%s</a></li>', get_comment_link($comment->comment_ID), $title_text, $comment->comment_author);
                            //Comma separator:
                            $commenters .= ($count +1 == (int)($post["comment_count"])) ? "" : ", ";
                        }
                    }
                }
                $tags_html .= sprintf('<ul class="recent-commentary-commenters-list">%s</ul>', $commenters);
                
            }
            $tags_html .= '</li>';
        }
        $tags_html .= "</ul>";
        return $tags_html;
    }
    
    public function build_raw($posts) {
        return $posts;
    }
    
    public function build_json($posts) {
        return "{ \"posts\": " . json_encode($posts) . "}";
    }
    
    public function build_src($posts, $varName="RecentCommentary") {
        return '<script type="text/javascript">var ' . $varName . ' = ' . self::build_json($posts) . ';</script>';
    }
    
    public function build_widget($posts) {
        $tags_html = $this->build_html($posts);
        extract($this->options["widget_args"]);
        $widget_html = sprintf("%s%s%s%s%s%s", $before_widget, $before_title, $this->options['title'], $after_title, $tags_html, $after_widget);
        return $widget_html;
    }
    
    //These are shortcut wrappers for the build_ functions that only works as
    //instance methods:
    public function get_html() {
        return $this->build_html($this->posts);
    }
    public function get_raw() {
        return $this->build_raw($this->posts);
    }
    public function get_json() {
        return $this->build_json($this->posts);
    }
    public function get_src() {
        return $this->build_src($this->posts);
    }
    public function get_widget() {
        return $this->build_widget($this->posts);
    }
    
    //Widget controls:
    function widget_control() {
        $default_options = array(
            'postlimit'  => 10,
            'commentlimit' => 10,
            'title' => "Recent Commentary",
            'show_on_single' => FALSE,
            'defaultcss' => FALSE
        );
        add_option('widget_tadhg_recent_commentary', $default_options);
        if ( isset($_POST["recent-commentary-submit"]) ) {
            $test = "!";
            $updated_options = array();
            //Assumes that all options have a key in $default_options
            //and that the key names are the same as the form input names with
            //a 'recent-commentary' prefix. Also sets to blank any values that
            //are in the defaults but not in the POST:
            foreach ($default_options as $key=>$value) {
                $postkey = sprintf("recent-commentary-%s", $key);
                $updated_options[$key] = strip_tags(stripslashes(self::pydictget($_POST, $postkey, "")));
            }
            update_option('widget_tadhg_recent_commentary', $updated_options);
            wp_tadhg_delete_recent_commentary_cache();
        }
    
        /* Get options */
        $options = array_merge($default_options, get_option('widget_tadhg_recent_commentary'));
        ?>
            <p>
                <label for="recent-commentary-title"><?php _e('Title:'); ?> <input class="widefat" id="recent-commentary-title" name="recent-commentary-title" type="text" value="<?php echo $options['title']; ?>" /></label>
            </p>
            <p>
                <label for="recent-commentary-postlimit"><?php _e('Number of posts to display:'); ?> <input style="width: 30px; text-align: center;" id="recent-commentary-postlimit" name="recent-commentary-postlimit" type="text" value="<?php echo $options['postlimit']; ?>" /></label>
            </p>
            <p>
                <label for="recent-commentary-commentlimit"><?php _e('Number of commenters to display below each post:'); ?> <input style="width: 30px; text-align: center;" id="recent-commentary-commentlimit" name="recent-commentary-commentlimit" type="text" value="<?php echo $options['commentlimit']; ?>" /></label>
            </p>
            <p>
                <?php _e('Show widget on single pages:'); ?><br />
                <input name="recent-commentary-show_on_single" id="recent-commentary-show_on_single-true" type="radio" value="TRUE" <?php if ($options['show_on_single']=="TRUE") {echo "checked=\"checked\"";} ?>>
                <label for="recent-commentary-show_on_single-true">Show</label>
                <input name="recent-commentary-show_on_single" id="recent-commentary-show_on_single-false" type="radio" value="FALSE" <?php if ($options['show_on_single']!="TRUE") {echo "checked=\"checked\"";} ?>
                <label for="recent-commentary-show_on_single-false">Hide</label>
            </p>
            <p>
                <label for="recent-commentary-defaultcss"><?php _e('Apply default CSS:'); ?> <input id="recent-commentary-defaultcss" name="recent-commentary-defaultcss" type="checkbox" value="TRUE" <?php $chk = $options['defaultcss'] ? 'checked=checked' : ''; echo($chk);?> /></label>
            </p>
            
            <input type="hidden" id="recent-commentary-submit" name="recent-commentary-submit" value="1" />
        <?php
    }
    
    //Helper functions
    function pydictget($arr, $key, $default) {
        //I just want something that does dict.get(key, default)!
        if (!array_key_exists($key, $arr)) {
            return $default;
        } else {
            return $arr[$key];
        }
        
    }
    
}

//Convenience functions for calling class functions:
function wp_tadhg_recent_commentary($args) {
    //Check for whether or not to show the widget depending on the page and
    //display settings:
    $admin_options = get_option('widget_tadhg_recent_commentary');
    if ($admin_options["show_on_single"]=="TRUE" || !is_single()) {
        $options = array(
            "mode"=>"widget",
            "widget_args"=>$args
        );
        $tadhgrecentcommentary = new TadhgRecentCommentary($options);
        echo($tadhgrecentcommentary->output);
    }
}

function wp_tadhg_recent_commentary_control() {
    TadhgRecentCommentary::widget_control();
}

//Cache deletion:
function wp_tadhg_delete_recent_commentary_cache() {
	wp_cache_delete( 'wp_tadhg_recent_commentary', 'widget' );
}

//Register function:
function wp_tadhg_recent_commentary_register() {
	$widget_ops = array('classname' => 'recent-commentary', 'description' => __( 'Posts with recent comments.' ) );
	if (function_exists('wp_register_sidebar_widget')) {
        wp_register_sidebar_widget('widget_tadhg_recent_commentary', __('Recent Commentary'), 'wp_tadhg_recent_commentary', $widget_ops);
    }
    if (function_exists('wp_register_widget_control')) {
        wp_register_widget_control('widget_tadhg_recent_commentary', __('Recent Commentary'), 'wp_tadhg_recent_commentary_control');
    }
}
//Hooks:
add_action('widgets_init', 'wp_tadhg_recent_commentary_register');
add_action('comment_post', 'wp_tadhg_delete_recent_commentary_cache');
add_action('wp_set_comment_status', 'wp_tadhg_delete_recent_commentary_cache');
?>
